@@ -1,27 +1,21 @@
-import sys
+import os
 from six.moves import builtins
 from six import assertRaisesRegex
 from io import BytesIO
-import os
+from unittest import TestCase
 
-from unittest import TestCase, skipUnless
 from Bio import SeqIO, bgzf
-from contextlib import contextmanager
 
 try:
-    from unittest.mock import patch
+    from unittest.mock import patch, mock_open
 except ImportError:
     from mock import patch
 
-from .mocking import mockOpen
-
 from dark.reads import Read, AARead, DNARead, RNARead, Reads
 from dark.fasta import (dedupFasta, dePrefixAndSuffixFasta, fastaSubtract,
-                        FastaReads, FastaFaiReads, combineReads, SqliteIndex)
+                        FastaReads, combineReads, SqliteIndex)
 from dark.utils import StringIO
 
-canTestPyfaidx = sys.platform != 'linux'
-
 
 class FastaDeDup(TestCase):
     """
@@ -268,8 +262,7 @@ class TestFastaReads(TestCase):
         """
         An empty FASTA file results in an empty iterator.
         """
-        mockOpener = mockOpen()
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open()):
             reads = FastaReads('filename.fasta')
             self.assertEqual([], list(reads))
 
@@ -278,8 +271,7 @@ class TestFastaReads(TestCase):
         A FASTA file with one read must be read properly.
         """
         data = '\n'.join(['>id1', 'ACGT'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(FastaReads('filename.fasta'))
             self.assertEqual([Read('id1', 'ACGT')], reads)
 
@@ -288,8 +280,7 @@ class TestFastaReads(TestCase):
         A FASTA file read must not have any quality information.
         """
         data = '\n'.join(['>id1', 'ACGT'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(FastaReads('filename.fasta'))
             self.assertEqual(None, reads[0].quality)
 
@@ -299,8 +290,7 @@ class TestFastaReads(TestCase):
         sequences must be returned in the correct order.
         """
         data = '\n'.join(['>id1', 'ACGT', '>id2', 'TGCA'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(FastaReads('filename.fasta'))
             self.assertEqual(2, len(reads))
             self.assertEqual([Read('id1', 'ACGT'), Read('id2', 'TGCA')], reads)
@@ -311,8 +301,7 @@ class TestFastaReads(TestCase):
         are instances of DNARead.
         """
         data = '\n'.join(['>id1', 'ACGT'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(FastaReads('filename.fasta'))
             self.assertTrue(isinstance(reads[0], DNARead))
 
@@ -322,8 +311,7 @@ class TestFastaReads(TestCase):
         are instances of AARead.
         """
         data = '\n'.join(['>id1', 'ACGST'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(FastaReads('filename.fasta', AARead))
             self.assertTrue(isinstance(reads[0], AARead))
 
@@ -333,8 +321,7 @@ class TestFastaReads(TestCase):
         are instances of DNARead.
         """
         data = '\n'.join(['>id1', 'ACGT'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(FastaReads('filename.fasta', DNARead))
             self.assertTrue(isinstance(reads[0], DNARead))
 
@@ -344,8 +331,7 @@ class TestFastaReads(TestCase):
         are instances of RNARead.
         """
         data = '\n'.join(['>id1', 'ACGT'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(FastaReads('filename.fasta', RNARead))
             self.assertTrue(isinstance(reads[0], RNARead))
 
@@ -354,8 +340,7 @@ class TestFastaReads(TestCase):
         A read needs to be converted from lower to upper case if specified.
         """
         data = '\n'.join(['>id1', 'actgs'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(FastaReads('filename.fasta', readClass=AARead,
                          upperCase=True))
             self.assertEqual([AARead('id1', 'ACTGS')], reads)
@@ -365,8 +350,7 @@ class TestFastaReads(TestCase):
         A read needs to be converted from lower to upper case if specified.
         """
         data = '\n'.join(['>id1', 'actg'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(FastaReads('filename.fasta', upperCase=True))
             self.assertEqual([AARead('id1', 'ACTG')], reads)
 
@@ -375,8 +359,7 @@ class TestFastaReads(TestCase):
         A read must not be converted from lower to upper case if not specified.
         """
         data = '\n'.join(['>id1', 'actgs'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(FastaReads('filename.fasta', readClass=AARead))
             self.assertEqual([AARead('id1', 'actgs')], reads)
 
@@ -387,8 +370,7 @@ class TestFastaReads(TestCase):
         C{trueLength} argument.
         """
         data = ''
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = FastaReads('filename.fasta')
             result = list(reads.filter(randomSubset=0, trueLength=0))
             self.assertEqual([], result)
@@ -400,8 +382,7 @@ class TestFastaReads(TestCase):
         C{trueLength} argument.
         """
         data = '\n'.join(['>id1', 'ACGT', '>id2', 'TGCA'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = FastaReads('filename.fasta')
             result = list(reads.filter(randomSubset=2, trueLength=2))
             self.assertEqual([Read('id1', 'ACGT'), Read('id2', 'TGCA')],
@@ -414,8 +395,7 @@ class TestFastaReads(TestCase):
         C{trueLength} argument.
         """
         data = '\n'.join(['>id', 'ACGT'] * 10)
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = FastaReads('filename.fasta')
             result = list(reads.filter(randomSubset=1, trueLength=10))
             self.assertEqual(1, len(result))
@@ -453,148 +433,6 @@ class TestFastaReads(TestCase):
                 list(reads))
 
 
-@skipUnless(canTestPyfaidx, 'pyfaidx tests are skipped on Linux')
-class TestFastaFaiReads(TestCase):
-    """
-    Tests for the L{dark.fasta.FastaFaiReads} class.
-    """
-    def testMissingKey(self):
-        """
-        If a non-existent sequence id is looked up, a KeyError must be raised.
-        """
-
-        pyfaidxIndex = StringIO()
-
-        class Open(object):
-            def __init__(self, test, manager):
-                self.test = test
-                self.manager = manager
-                self.count = 0
-
-            def sideEffect(self, filename, *args, **kwargs):
-                if self.count == 0:
-                    self.test.assertEqual('filename.fasta', filename)
-                    self.count += 1
-                    return BytesIO(b'>id1\nACTG\n')
-                elif self.count == 1:
-                    self.test.assertEqual('filename.fasta', filename)
-                    self.count += 1
-                    return StringIO('>id1\nACTG\n')
-                elif self.count == 2:
-                    self.count += 1
-                    return self.manager
-                elif self.count == 3:
-                    self.count += 1
-                    return StringIO(pyfaidxIndex.getvalue())
-                else:
-                    self.test.fail(
-                        'Open called too many times. Filename: %r, Args: %r, '
-                        'Keyword args: %r.' % (filename, args, kwargs))
-
-        @contextmanager
-        def manager():
-            yield pyfaidxIndex
-
-        sideEffect = Open(self, manager()).sideEffect
-        with patch.object(builtins, 'open') as mockMethod:
-            mockMethod.side_effect = sideEffect
-            reads = FastaFaiReads('filename.fasta')
-            error = "^'id2 not in filename\\.fasta\\.'"
-            assertRaisesRegex(self, KeyError, error, reads.__getitem__, 'id2')
-
-    def testOneRead(self):
-        """
-        It must be possible to access a FASTA file with one read like a dict.
-        """
-
-        pyfaidxIndex = StringIO()
-
-        class Open(object):
-            def __init__(self, test, manager):
-                self.test = test
-                self.manager = manager
-                self.count = 0
-
-            def sideEffect(self, filename, *args, **kwargs):
-                if self.count == 0:
-                    self.test.assertEqual('filename.fasta', filename)
-                    self.count += 1
-                    return BytesIO(b'>id1\nACTG\n')
-                elif self.count == 1:
-                    self.test.assertEqual('filename.fasta', filename)
-                    self.count += 1
-                    return StringIO('>id1\nACTG\n')
-                elif self.count == 2:
-                    self.count += 1
-                    return self.manager
-                elif self.count == 3:
-                    self.count += 1
-                    return StringIO(pyfaidxIndex.getvalue())
-                else:
-                    self.test.fail(
-                        'Open called too many times. Filename: %r, Args: %r, '
-                        'Keyword args: %r.' % (filename, args, kwargs))
-
-        @contextmanager
-        def manager():
-            yield pyfaidxIndex
-
-        sideEffect = Open(self, manager()).sideEffect
-        with patch.object(builtins, 'open') as mockMethod:
-            mockMethod.side_effect = sideEffect
-            reads = FastaFaiReads('filename.fasta')
-            self.assertEqual(DNARead('id1', 'ACTG'), reads['id1'])
-            # Check that the fai index was built correctly.
-            self.assertEqual(pyfaidxIndex.getvalue(), 'id1\t4\t5\t4\t5\n')
-
-    def testTwoReads(self):
-        """
-        It must be possible to access a FASTA file with two reads like a dict.
-        """
-
-        pyfaidxIndex = StringIO()
-
-        class Open(object):
-            def __init__(self, test, manager):
-                self.test = test
-                self.manager = manager
-                self.count = 0
-
-            def sideEffect(self, filename, *args, **kwargs):
-                if self.count == 0:
-                    self.test.assertEqual('filename.fasta', filename)
-                    self.count += 1
-                    return BytesIO(b'>id1\nACTG\n>id2\nAACCTTGG\n')
-                elif self.count == 1:
-                    self.test.assertEqual('filename.fasta', filename)
-                    self.count += 1
-                    return StringIO('>id1\nACTG\n>id2\nAACCTTGG\n')
-                elif self.count == 2:
-                    self.count += 1
-                    return self.manager
-                elif self.count == 3:
-                    self.count += 1
-                    return StringIO(pyfaidxIndex.getvalue())
-                else:
-                    self.test.fail(
-                        'Open called too many times. Filename: %r, Args: %r, '
-                        'Keyword args: %r.' % (filename, args, kwargs))
-
-        @contextmanager
-        def manager():
-            yield pyfaidxIndex
-
-        sideEffect = Open(self, manager()).sideEffect
-        with patch.object(builtins, 'open') as mockMethod:
-            mockMethod.side_effect = sideEffect
-            reads = FastaFaiReads('filename.fasta')
-            self.assertEqual(DNARead('id1', 'ACTG'), reads['id1'])
-            self.assertEqual(DNARead('id2', 'AACCTTGG'), reads['id2'])
-            # Check that the fai index was built correctly.
-            self.assertEqual(pyfaidxIndex.getvalue(),
-                             'id1\t4\t5\t4\t5\nid2\t8\t15\t8\t9\n')
-
-
 class TestCombineReads(TestCase):
     """
     Tests for the L{dark.fasta.combineReads} function.
@@ -621,8 +459,7 @@ class TestCombineReads(TestCase):
         FastaReads must contain the expected read.
         """
         data = '\n'.join(['>id1', 'ACGT'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(combineReads('filename.fasta', None))
             self.assertEqual([Read('id1', 'ACGT')], reads)
 
@@ -632,8 +469,7 @@ class TestCombineReads(TestCase):
         FastaReads must contain the expected read, in the original case.
         """
         data = '\n'.join(['>id1', 'AcgT'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(combineReads('filename.fasta', None))
             self.assertEqual([Read('id1', 'AcgT')], reads)
 
@@ -643,8 +479,7 @@ class TestCombineReads(TestCase):
         FastaReads must have the read sequence in uppper case.
         """
         data = '\n'.join(['>id1', 'acgt'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(combineReads('filename.fasta', None, upperCase=True))
             self.assertEqual([Read('id1', 'ACGT')], reads)
 
@@ -698,8 +533,7 @@ class TestCombineReads(TestCase):
         specified sequences.
         """
         data = '\n'.join(['>id1', 'ACGT'])
-        mockOpener = mockOpen(read_data=data)
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open(read_data=data)):
             reads = list(combineReads('filename.fasta', ['ACGT'],
                                       readClass=RNARead))
             self.assertTrue(isinstance(reads[0], RNARead))
